@@ -20,6 +20,8 @@
  *
  */
 
+#include "config.h"
+
 #include <net/grl-net.h>
 #include <string.h>
 #include <errno.h>
@@ -32,6 +34,10 @@
 #include "lua-library/lua-libraries.h"
 #include "lua-library/htmlentity.h"
 
+#ifdef HAVE_TOTEM_PLPARSER_MINI
+#include <totem-pl-parser-mini.h>
+#endif
+
 #define GRL_LOG_DOMAIN_DEFAULT lua_library_log_domain
 GRL_LOG_DOMAIN_STATIC (lua_library_log_domain);
 
@@ -65,6 +71,24 @@ char_str (gunichar c,
   return buf;
 }
 
+/* ANSI HTML entities
+ * http://www.w3schools.com/charsets/ref_html_ansi.asp */
+static gchar *
+ansi_char_str (gunichar c,
+               gchar   *buf)
+{
+  gchar from_c[2], *tmp;
+
+  memset (buf, 0, 8);
+  from_c[0] = c;
+  from_c[1] = '\0';
+  tmp = g_convert (from_c, 2, "UTF-8", "Windows-1252", NULL, NULL, NULL);
+  strcpy (buf, tmp);
+  g_free (tmp);
+
+  return buf;
+}
+
 /* Adapted from unescape_gstring_inplace() in gmarkup.c in glib */
 static char *
 unescape_string (const char *orig_from)
@@ -116,7 +140,10 @@ unescape_string (const char *orig_from)
             (0xE000 <= l && l <= 0xFFFD) ||
             (0x10000 <= l && l <= 0x10FFFF)) {
           gchar buf[8];
-          char_str (l, buf);
+          if (l >= 128 && l <= 255)
+            ansi_char_str (l, buf);
+          else
+            char_str (l, buf);
           strcpy (to, buf);
           to += strlen (buf) - 1;
           from = end;
@@ -153,7 +180,7 @@ grl_data_set_lua_string (GrlData    *data,
   if (g_utf8_validate (str, -1, NULL) == FALSE) {
     fixed = g_convert (str, -1, "UTF-8", "ISO8859-1", NULL, NULL, NULL);
     if (fixed == NULL) {
-      g_warning ("Ignored non-UTF-8 and non-ISO8859-1 string for field '%s'", key_name);
+      GRL_WARNING ("Ignored non-UTF-8 and non-ISO8859-1 string for field '%s'", key_name);
       return;
     }
   }
@@ -166,6 +193,56 @@ grl_data_set_lua_string (GrlData    *data,
   }
 }
 
+static void
+grl_data_add_lua_string (GrlData    *data,
+                         GrlKeyID    key_id,
+                         const char *key_name,
+                         const char *str)
+{
+  char *fixed = NULL;
+
+  /* Check for UTF-8 or ISO8859-1 string */
+  if (g_utf8_validate (str, -1, NULL) == FALSE) {
+    fixed = g_convert (str, -1, "UTF-8", "ISO8859-1", NULL, NULL, NULL);
+    if (fixed == NULL) {
+      GRL_WARNING ("Ignored non-UTF-8 and non-ISO8859-1 string for field '%s'", key_name);
+      return;
+    }
+  }
+
+  if (fixed) {
+    grl_data_add_string (data, key_id, fixed);
+    g_free (fixed);
+  } else {
+    grl_data_add_string (data, key_id, str);
+  }
+}
+
+static gboolean
+verify_plaintext_fetch (GrlSource  *source,
+                        char      **urls,
+                        guint       num_urls)
+{
+  const char **tags;
+  gboolean has_plaintext_tag;
+  guint i;
+
+  tags = grl_source_get_tags (source);
+  has_plaintext_tag = (tags && g_strv_contains (tags, "net:plaintext"));
+
+  /* No need to verify the URLs, the source is saying that they do
+   * plaintext queries, so nothing for us to block */
+  if (has_plaintext_tag)
+    return TRUE;
+
+  for (i = 0; i < num_urls; i++) {
+    if (g_str_has_prefix (urls[i], "http:"))
+      return FALSE;
+  }
+
+  return TRUE;
+}
+
 /* Top of the stack must be a table */
 static void
 grl_util_add_table_to_media (lua_State *L,
@@ -192,6 +269,11 @@ grl_util_add_table_to_media (lua_State *L,
         grl_data_add_int (GRL_DATA (media), key_id, lua_tointeger (L, -1));
       break;
 
+    case G_TYPE_INT64:
+      if (lua_isnumber (L, -1))
+        grl_data_add_int64 (GRL_DATA (media), key_id, lua_tointeger (L, -1));
+      break;
+
     case G_TYPE_FLOAT:
       if (lua_isnumber (L, -1))
         grl_data_add_float (GRL_DATA (media), key_id, lua_tointeger (L, -1));
@@ -199,7 +281,7 @@ grl_util_add_table_to_media (lua_State *L,
 
     case G_TYPE_STRING:
       if (lua_isstring (L, -1))
-        grl_data_set_lua_string (GRL_DATA (media), key_id, key_name, lua_tostring (L, -1));
+        grl_data_add_lua_string (GRL_DATA (media), key_id, key_name, lua_tostring (L, -1));
       break;
 
     default:
@@ -287,6 +369,17 @@ grl_util_build_media (lua_State *L,
         }
         break;
 
+      case G_TYPE_INT64:
+        if (lua_isnumber (L, -1)) {
+          grl_data_set_int64 (GRL_DATA (media), key_id, lua_tointeger (L, -1));
+        } else if (lua_istable (L, -1)) {
+          grl_util_add_table_to_media (L, media, key_id, key_name, type);
+        } else if (!lua_isnil (L, -1)) {
+          GRL_WARNING ("'%s' is not compatible for '%s'",
+                       lua_typename (L, -1), key_name);
+        }
+        break;
+
       case G_TYPE_STRING:
         if (lua_isstring (L, -1)) {
           grl_data_set_lua_string (GRL_DATA (media), key_id, key_name, lua_tostring (L, -1));
@@ -313,11 +406,17 @@ grl_util_build_media (lua_State *L,
           GDateTime *date;
           const char *date_str = lua_tostring (L, -1);
           date = grl_date_time_from_iso8601 (date_str);
+          /* Try a number of seconds since Epoch */
+          if (!date) {
+            gint64 date_int = g_ascii_strtoll (date_str, NULL, 0);
+            if (date_int)
+              date = g_date_time_new_from_unix_utc (date_int);
+          }
           if (date) {
             grl_data_set_boxed (GRL_DATA (media), key_id, date);
             g_date_time_unref (date);
           } else {
-            GRL_WARNING ("'%s' is not a valid ISO-8601 date", date_str);
+            GRL_WARNING ("'%s' is not a valid ISO-8601 or Epoch date", date_str);
           }
         } else if (type == G_TYPE_BYTE_ARRAY) {
            gsize size = luaL_len (L, -1);
@@ -351,17 +450,25 @@ grl_util_fetch_done (GObject *source_object,
   OperationSpec *os;
   FetchOperation *fo = (FetchOperation *) user_data;
   lua_State *L = fo->L;
+  gchar *fixed = NULL;
 
   if (!grl_net_wc_request_finish (GRL_NET_WC (source_object),
                                   res, &data, &len, &err)) {
     data = NULL;
   } else if (!g_utf8_validate(data, len, NULL)) {
-    data = NULL;
-    g_set_error_literal (&err, G_IO_ERROR, G_IO_ERROR_INVALID_DATA,
-                         "Fetched item is not valid UTF-8");
+    fixed = g_convert (data, len, "UTF-8", "ISO8859-1", NULL, NULL, NULL);
+    if (!fixed) {
+      g_set_error_literal (&err, G_IO_ERROR, G_IO_ERROR_INVALID_DATA,
+                           "Fetched item is not valid UTF-8 or ISO8859-1");
+      data = NULL;
+    } else {
+      data = fixed;
+    }
   }
 
   fo->results[fo->index] = (err == NULL) ? g_strdup (data) : g_strdup ("");
+  g_free (fixed);
+
   if (err != NULL) {
     GRL_WARNING ("Can't fetch element %d (URL: %s): '%s'", fo->index + 1, fo->url, err->message);
     g_error_free (err);
@@ -458,7 +565,7 @@ get_zipped_contents (guchar        *data,
 
     if (r != ARCHIVE_OK) {
       if (r != ARCHIVE_EOF && r == ARCHIVE_FATAL)
-        g_warning ("Fatal error handling archive: %s", archive_error_string (a));
+        GRL_WARNING ("Fatal error handling archive: %s", archive_error_string (a));
       break;
     }
 
@@ -474,9 +581,9 @@ get_zipped_contents (guchar        *data,
       if (read <= 0) {
         g_free (buf);
         if (read < 0)
-          g_warning ("Fatal error reading '%s' in archive: %s", name, archive_error_string (a));
+          GRL_WARNING ("Fatal error reading '%s' in archive: %s", name, archive_error_string (a));
         else
-          g_warning ("Read an empty file from the archive");
+          GRL_WARNING ("Read an empty file from the archive");
       } else {
         GRL_DEBUG ("Setting content for %s at %d", name, idx);
         /* FIXME check for validity? */
@@ -555,7 +662,7 @@ net_wc_new_with_options(lua_State *L,
   GrlNetWc *wc;
 
   wc = grl_net_wc_new ();
-  if (arg_offset < lua_gettop (L) && lua_istable (L, arg_offset)) {
+  if (arg_offset <= lua_gettop (L) && lua_istable (L, arg_offset)) {
     /* Set GrlNetWc options */
     lua_pushnil (L);
     while (lua_next (L, arg_offset) != 0) {
@@ -699,8 +806,8 @@ grl_l_operation_get_options (lua_State *L)
     const gchar *key_name = NULL;
     GrlRegistry *registry = grl_registry_get_default ();
 
-    luaL_argcheck (L, lua_isstring (L, 3), 3, "expecting key name");
-    key_name = lua_tostring (L, 3);
+    luaL_argcheck (L, lua_isstring (L, 2), 2, "expecting key name");
+    key_name = lua_tostring (L, 2);
 
     key = grl_registry_lookup_metadata_key (registry, key_name);
     if (key != GRL_METADATA_KEY_INVALID) {
@@ -918,6 +1025,14 @@ grl_l_fetch (lua_State *L)
     }
   }
 
+  if (!verify_plaintext_fetch (os->source, urls, num_urls)) {
+    GRL_WARNING ("Source '%s' is broken, it makes plaintext network queries but "
+                 "does not set the 'net:plaintext' tag", grl_source_get_id (os->source));
+    g_free (urls);
+    os->pending_ops--;
+    return 1;
+  }
+
   lua_callback = lua_tolstring (L, 2, NULL);
 
   wc = net_wc_new_with_options(L, 3);
@@ -963,7 +1078,11 @@ grl_l_callback (lua_State *L)
 
   nparam = lua_gettop (L);
   os = grl_lua_library_get_current_operation (L);
-  g_return_val_if_fail (os != NULL, 0);
+  if (os == NULL) {
+    luaL_error (L, "Source is broken as grl.callback() was called "
+                "after last operation has been finalized");
+    return 0;
+  }
 
   media = (os->op_type == LUA_RESOLVE) ? os->media : NULL;
   if (nparam > 0) {
@@ -1058,6 +1177,29 @@ grl_l_dgettext (lua_State *L)
   return 1;
 }
 
+/**
+ * grl.encode
+ *
+ * @text: (string) the string to %-encode
+ * @return: the %-encoded version of @text
+ */
+static gint
+grl_l_encode (lua_State *L)
+{
+  const gchar *text;
+  gchar *output;
+
+  luaL_argcheck (L, lua_isstring (L, 1), 1, "expecting part as string");
+
+  text = lua_tolstring (L, 1, NULL);
+
+  output = g_uri_escape_string (text, NULL, FALSE);
+  lua_pushstring (L, output);
+  g_free (output);
+
+  return 1;
+}
+
 /**
  * grl.decode
  *
@@ -1167,8 +1309,142 @@ grl_l_unzip (lua_State *L)
   return 1;
 }
 
+/**
+ * grl.goa_consumer_key
+ *
+ * Only available for gnome-online-accounts sources.
+ *
+ * @return: The consumer key for the configured account or
+ * nil if gnome-online-accounts isn't available.
+ */
+static gint
+grl_l_goa_consumer_key (lua_State *L)
+{
+#ifndef GOA_ENABLED
+  GRL_WARNING ("Source '%s' is broken as it tries to access gnome-online-accounts "
+               "information, but it should not have been created");
+  return 0;
+#else
+  {
+    GoaObject *object;
+    GoaOAuth2Based *oauth2 = NULL;
+
+    object = grl_lua_library_load_goa_data (L);
+    if (object != NULL) {
+      /* FIXME handle other types of object? */
+      oauth2 = goa_object_peek_oauth2_based (object);
+    }
+    if (oauth2 == NULL) {
+      GRL_WARNING ("Source is broken as it tries to access gnome-online-accounts "
+                   "information, but it doesn't declare what account data it needs, or"
+                   "the account type is not supported.");
+      lua_pushnil (L);
+      return 1;
+    } else {
+      lua_pushstring (L, goa_oauth2_based_get_client_id (GOA_OAUTH2_BASED (oauth2)));
+      return 1;
+    }
+  }
+#endif
+}
+
+/**
+ * grl.goa_access_token
+ *
+ * Only available for gnome-online-accounts sources.
+ *
+ * @return: The access token for the configured account or
+ * nil if gnome-online-accounts isn't available.
+ */
+static gint
+grl_l_goa_access_token (lua_State *L)
+{
+#ifndef GOA_ENABLED
+  GRL_WARNING ("Source '%s' is broken as it tries to access gnome-online-accounts "
+               "information, but it should not have been created",
+               grl_source_get_id (os->source));
+  return 0;
+#else
+  {
+    GoaObject *object;
+    GoaOAuth2Based *oauth2 = NULL;
+
+    object = grl_lua_library_load_goa_data (L);
+
+    if (object != NULL) {
+      /* FIXME handle other types of object? */
+      oauth2 = goa_object_peek_oauth2_based (object);
+    }
+    if (oauth2 == NULL) {
+      GRL_WARNING ("Source is broken as it tries to access gnome-online-accounts "
+                   "information, but it doesn't declare what account data it needs, or "
+                   "the account type is not supported.");
+      lua_pushnil (L);
+      return 1;
+    } else {
+      gchar *access_token;
+
+      goa_oauth2_based_call_get_access_token_sync (oauth2,
+                                                   &access_token,
+                                                   NULL, NULL, NULL);
+      lua_pushstring (L, access_token);
+      g_free (access_token);
+      return 1;
+    }
+  }
+#endif
+}
+
+static gint
+grl_l_is_video_site (lua_State *L)
+{
+  const char *url;
+  gboolean ret = FALSE;
+
+  luaL_argcheck (L, lua_isstring (L, 1), 1,
+                 "expecting url as string");
+
+  url = lua_tolstring (L, 1, NULL);
+
+#ifdef HAVE_TOTEM_PLPARSER_MINI
+  ret = totem_pl_parser_can_parse_from_uri (url, FALSE);
+#else
+  GRL_DEBUG ("Return FALSE for whether '%s' is a video site, as compiled without totem-plparser-mini support", url);
+#endif
+
+  lua_pushboolean (L, ret);
+
+  return 1;
+}
+
 /* ================== Lua-Library initialization =========================== */
 
+/** Load library included as GResource and run it with lua_pcall.
+  * Caller should handle the stack aftwards.
+ **/
+static gboolean
+load_gresource_library (lua_State   *L,
+                        const gchar *uri)
+{
+  GFile *file;
+  gchar *data;
+  gsize size;
+  GError *error = NULL;
+  gboolean ret = TRUE;
+
+  file = g_file_new_for_uri (uri);
+  g_file_load_contents (file, NULL, &data, &size, NULL, &error);
+  g_assert_no_error (error);
+  g_clear_pointer (&file, g_object_unref);
+
+  if (luaL_dostring (L, data)) {
+    GRL_WARNING ("Failed to load %s due %s", uri, lua_tostring (L, -1));
+    ret = FALSE;
+  }
+  g_free (data);
+  return ret;
+}
+
 gint
 luaopen_grilo (lua_State *L)
 {
@@ -1181,9 +1457,13 @@ luaopen_grilo (lua_State *L)
     {"debug", &grl_l_debug},
     {"warning", &grl_l_warning},
     {"dgettext", &grl_l_dgettext},
+    {"encode", &grl_l_encode},
     {"decode", &grl_l_decode},
     {"unescape", &grl_l_unescape},
     {"unzip", &grl_l_unzip},
+    {"goa_access_token", &grl_l_goa_access_token},
+    {"goa_consumer_key", &grl_l_goa_consumer_key},
+    {"is_video_site", &grl_l_is_video_site},
     {NULL, NULL}
   };
 
@@ -1200,6 +1480,23 @@ luaopen_grilo (lua_State *L)
   luaopen_json (L);
   lua_settable (L, -3);
 
+  /* Load inspect.lua and save object in global environment table */
+  lua_getglobal (L, LUA_ENV_TABLE);
+  if (load_gresource_library (L, URI_LUA_LIBRARY_INSPECT) &&
+      lua_istable (L, -1)) {
+      /* Top of the stack is inspect table from inspect.lua */
+      lua_getfield (L, -1, "inspect");
+      /* Top of the stack is inspect.inspect */
+      lua_setfield (L, -4, GRILO_LUA_LIBRARY_INSPECT);
+      /* grl.lua.inspect points to inspect.inspect */
+
+      /* Save inspect table in LUA_ENV_TABLE */
+      lua_setfield (L, -2, GRILO_LUA_INSPECT_INDEX);
+  } else {
+      GRL_WARNING ("Failed to load inspect.lua");
+  }
+  lua_pop (L, 1);
+
   /* Those modules are called in 'lua' table, inside 'grl' */
   lua_settable (L, -3);
   return 1;
@@ -1301,8 +1598,8 @@ grl_lua_library_set_current_operation (lua_State *L, guint operation_id)
   os = grl_lua_library_get_current_operation (L);
   if (os) {
     if (os->pending_ops == 0 && !os->callback_done) {
-      g_warning ("Source '%s' is broken, as there are no pending operations "
-                 "and grl.callback() was not called", grl_source_get_id (os->source));
+      GRL_WARNING ("Source '%s' is broken, as there are no pending operations "
+                   "and grl.callback() was not called", grl_source_get_id (os->source));
       switch (os->op_type) {
       case LUA_RESOLVE:
         os->cb.resolve (os->source, os->operation_id, NULL, os->user_data, NULL);
@@ -1346,3 +1643,49 @@ grl_lua_library_get_current_operation (lua_State *L)
 
   return os;
 }
+
+/**
+ * grl_lua_library_save_goa_data
+ *
+ * @L: LuaState where the data will be stored.
+ * @goa_object: #GoaObject to store.
+ * @return: Nothing.
+ *
+ * Stores the GoaObject from Lua-Factory in the global environment of
+ * lua_State.
+ **/
+void
+grl_lua_library_save_goa_data (lua_State *L, gpointer goa_object)
+{
+  g_return_if_fail (goa_object != NULL);
+
+#ifdef GOA_ENABLED
+  lua_pushlightuserdata (L, goa_object);
+  lua_setglobal (L, GOA_LUA_NAME);
+#else
+  GRL_WARNING ("grl_lua_library_save_goa_data() called but GOA support disabled.");
+#endif /* GOA_ENABLED */
+}
+
+/**
+ * grl_lua_library_load_goa_data
+ *
+ * @L: LuaState where the data is stored.
+ * @return: The #GoaObject.
+ **/
+gpointer
+grl_lua_library_load_goa_data (lua_State *L)
+{
+#ifdef GOA_ENABLED
+  GoaObject *goa_object;
+
+  lua_getglobal (L, GOA_LUA_NAME);
+  goa_object = (lua_islightuserdata(L, -1)) ? lua_touserdata(L, -1) : NULL;
+  lua_pop(L, 1);
+
+  return goa_object;
+#else
+  GRL_WARNING ("grl_lua_library_load_goa_data() called but GOA support disabled.");
+  return NULL;
+#endif /* GOA_ENABLED */
+}
